@@ -74,6 +74,8 @@ impl Storage for ProgramData {
                     e.get_mut().used_as_callee |= var_info.used_as_callee;
                     e.get_mut().used_as_arg |= var_info.used_as_arg;
 
+                    e.get_mut().pure_fn |= var_info.pure_fn;
+
                     if !var_info.is_fn_local {
                         e.get_mut().is_fn_local = false;
                     }
@@ -279,4 +281,8 @@ impl VarDataLike for VarUsageInfo {
     fn mark_initialized_with_safe_value(&mut self) {
         self.no_side_effect_for_member_access = true;
     }
+
+    fn mark_as_pure_fn(&mut self) {
+        self.pure_fn = true;
+    }
 }
